@@ -23,8 +23,6 @@
 #include <stdbool.h>
 #include <string.h>
 
-#include <osmocom/core/bitvec.h>
-#include <osmocom/core/utils.h>
 #include <osmocom/codec/codec.h>
 
 /* GSM HR unvoiced (mode=0) frames - subjective importance bit ordering */
@@ -265,26 +263,68 @@ const uint16_t gsm620_voiced_bitorder[112] = {
 	81,	/* Code 3:7 */
 };
 
+/*
+ * There is no officially defined silence frame for GSM-HR codec like there is
+ * for GSM-FR.  However, if one feeds all-zeros (complete silence) linear PCM
+ * input to the official GSM-HR encoder, the result will be an endless stream
+ * of these frames:
+ *
+ * R0=00 LPC=164,171,cb Int=0 Mode=0
+ * s1=00,00,00 s2=00,00,00 s3=00,00,00 s4=00,00,00
+ *
+ * The following const datum is the above unofficial GSM-HR silence frame in
+ * the packed RTP format of TS 101 318.
+ */
+const uint8_t osmo_gsm620_silence_frame[GSM_HR_BYTES] = {
+	0x01, 0x64, 0xB8, 0xE5, 0x80, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
+};
+
+/*
+ * GSM 06.20 defines, by reference to GSM 06.06 C code and GSM 06.07 test
+ * sequences, a special frame of codec parameters called the decoder homing
+ * frame (DHF).  When a spec-compliant speech decoder processes this frame,
+ * it resets itself to the spec-defined home state.
+ *
+ * The following const datum is GSM-HR DHF in the packed RTP format of
+ * TS 101 318.
+ */
+const uint8_t osmo_gsm620_homing_frame[GSM_HR_BYTES] = {
+	0x03, 0x71, 0xAF, 0x61, 0xC8, 0xF2, 0x80,
+	0x25, 0x31, 0xC0, 0x00, 0x00, 0x00, 0x00
+};
+
 /*! Check whether RTP frame contains HR SID code word according to
  *  TS 101 318 §5.2.2
  *  \param[in] rtp_payload Buffer with RTP payload
  *  \param[in] payload_len Length of payload
  *  \returns true if code word is found, false otherwise
+ *
+ *  Note that this function checks only for a perfect, error-free SID.
+ *  Unlike GSM 06.31 for FR or GSM 06.81 for EFR, GSM 06.41 spec for HR
+ *  does not prescribe exact bit counting rules, hence detection of
+ *  partially corrupted SID frames in downstream network elements
+ *  without out-of-band indication is not possible.
  */
 bool osmo_hr_check_sid(const uint8_t *rtp_payload, size_t payload_len)
 {
-	struct bitvec bv = {
-		.data = (uint8_t *)rtp_payload,
-		.data_len = payload_len,
-	};
+	static const uint8_t all_ff_bytes[9] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
+						0xFF, 0xFF, 0xFF, 0xFF};
 
-	/* A SID frame is identified by a SID codeword consisting of 79 bits which are all 1,
-	 * so we basically check if all bits in range r34..r112 (inclusive) are 1. */
-	for (bv.cur_bit = 33; bv.cur_bit < bv.data_len * 8; bv.cur_bit++)
-		if (bitvec_get_bit_pos(&bv, bv.cur_bit) != ONE)
-			return false;
+	if (payload_len < GSM_HR_BYTES)
+		return false;
 
-	return true;
+	/* A SID frame is identified by a SID codeword consisting of 79 bits
+	 * which are all 1, so we basically check if all bits in range
+	 * r34..r112 (inclusive) are 1.  However, given the position of
+	 * these bits in the frame, the most efficient way to perform
+	 * this check does not use any bit-level operations. */
+	if ((rtp_payload[4] & 0x7F) != 0x7F)
+		return false;
+	if (memcmp(rtp_payload + 5, all_ff_bytes, 9) == 0)
+		return true;
+	else
+		return false;
 }
 
 /*! Reset the SID field of a potentially corrupted, but still valid GSM-HR
